/* ========================================
 *
 * GT-521F32 
 * Optical Fingerprint Recognition EMBEDDED module
 * 
*/
#include "project.h"
#include <stdio.h>
#include "GT521F32.h"
#include "GT521F32_Defs.h"

extern char     str[] ; /* print buf */
uint8_t data_buf[512] ; /* data buffer */

#define TIMEOUT_COUNT 1000


static int enroll_without_save = 0 ;

Command_Packet_Type _cmd_packet = {
    0xAA55,
    0x0001,     /* device id, default = 0x0001, always fixed */
    0x00000000, /* input parameter DWORD */
    0x0000,     /* Command Code */
    0x0000      /* Check Sum */
} ;

Response_Packet_Type _res_packet = {
    0xAA55,
    0x0001,     /* device id, default = 0x0001, always fixed */
    0x00000000, /* parameter if ACK, Output Parameter, if Nack Error Code */
    0x0000,     /* Response, 0x30(ACK) 0x31(NACK) */
    0x0000      /* Check Sum */
} ;

Data_Packet_Type _data_packet = {
    0xA55A,
    0x0001,     /* device id, default 0x0001, always fixed */
    0,          /* pointer to N bytes data array */
    0x0000      /* Check Sum */
} ;

uint16_t fps_send_byte(uint8_t theByte) 
{
    FPS_PutChar(theByte) ;
    return( theByte ) ;
}

uint16_t fps_send_word(uint16_t theWord)
{
    uint16_t  sum = 0 ;
    sum += fps_send_byte( theWord       & 0xFF) ;
    sum += fps_send_byte((theWord >> 8) & 0xFF) ;
    return( sum ) ;
}

uint16_t fps_send_dword(uint32_t theDword)
{
    uint16_t sum = 0 ;
    sum += fps_send_byte( theDword        & 0xFF) ;
    sum += fps_send_byte((theDword >>  8) & 0xFF) ;
    sum += fps_send_byte((theDword >> 16) & 0xFF) ;
    sum += fps_send_byte((theDword >> 24) & 0xFF) ;
    return( sum ) ;
}

uint8_t fps_recv_byte(void)
{
    uint8_t theByte = 0 ;
    uint32_t timeout = TIMEOUT_COUNT ;
    
    while(FPS_GetRxBufferSize() == 0) {
        CyDelay(1) ;
        timeout-- ;
        if (timeout == 0) {
            break ;
        }
    }
    if (timeout == 0) {
        UART_PutString("Recv Byte Timedout!\n") ;
    }
    theByte = FPS_GetByte() ;
    return( theByte ) ;
}

uint16_t fps_recv_word(uint16_t *theWord)
{
    uint8_t aByte = 0 ;
    uint16_t sum = 0 ;
    int i ;
    uint32_t timeout = TIMEOUT_COUNT ;
    while(FPS_GetRxBufferSize() < 2) {
        CyDelay(1) ;
        timeout-- ;
        if (timeout == 0) {
            break ;
        }
    }
    if (timeout == 0) {
        UART_PutString("Recv Word Timedout!\n") ;
    }
    *theWord = 0 ;

    for (i = 0 ; i < 2 ; i++ ) {
        aByte = FPS_GetByte() ;
        sum += aByte ;
        *theWord |= ((aByte & 0xFF) << (8 * i)) ;
    }
    return( sum ) ;
}

uint16_t fps_recv_dword(uint32_t *theDword)
{
    uint8_t aByte = 0 ;
    uint16_t sum = 0 ;
    int i ;
    uint32_t timeout = TIMEOUT_COUNT ;
    
    while(FPS_GetRxBufferSize() < 2) {
        CyDelay(1) ;
        timeout-- ;
        if (timeout == 0) {
            break ;
        }
    }
    if (timeout == 0) {
        UART_PutString("Recv Dword Timedout!\n") ;
    }
    *theDword = 0 ;
    for (i = 0 ; i < 4 ; i++ ) {
        aByte = FPS_GetByte() ;
        sum += aByte ;
        *theDword |= ((aByte & 0xFF) << (8 * i)) ;
    }    
    return( sum ) ;
}

uint16_t fps_recv_data(uint8_t data[], int num_data)
{
    uint16_t sum = 0 ;
    int i ;
    uint32_t timeout = TIMEOUT_COUNT ;
    for (i = 0 ; i < num_data ; i++ ) {
        timeout = TIMEOUT_COUNT ;
        while(FPS_GetRxBufferSize() == 0) {
            CyDelay(1) ;
            timeout-- ;
            if (timeout == 0) {
                break ;
            }
        }
        if (timeout == 0) {
            sprintf(str, "Recv data[%d] Timedout!\n", i) ;
            UART_PutString(str) ;
        }
        data[i] = FPS_GetByte() ;
        sum += data[i] ;
    }
    return( sum ) ;
}

uint16_t fps_recv_and_dump_data(int num_data)
{
    uint16_t sum = 0 ;
    int i ;
    uint8_t aByte = 0 ;
    uint32_t timeout = TIMEOUT_COUNT ;
    for (i = 0 ; i < num_data ; i++ ) {
        timeout = TIMEOUT_COUNT ;
        while(FPS_GetRxBufferSize() == 0) {
            CyDelay(1) ;
            timeout-- ;
            if (timeout == 0) {
                break ;
            }
        }
        if (timeout == 0) {
            sprintf(str, "Recv data[%d] Timedout!\n", i) ;
            UART_PutString(str) ;
        }
        aByte = FPS_GetByte() ;
        sum += aByte ;
        sprintf(str, "%02X ", aByte) ;
        UART_PutString(str) ;
        if (((i+1)%0x10) == 0x00) {
            UART_PutString("\n") ;
        }
    }
    if (i % 0x10) {
        UART_PutString("\n") ;
    }
    return( sum ) ;
}

int fps_send_command_packet(Command_Packet_Type *cmd) 
{
    int result = 1 ; /* 0 for success */
    if (cmd != 0) {
        cmd->check_sum = 0 ;
        cmd->check_sum += fps_send_word(cmd->head) ;
        cmd->check_sum += fps_send_word(cmd->device_id) ;
        cmd->check_sum += fps_send_dword(cmd->parameter) ;
        cmd->check_sum += fps_send_word(cmd->command) ;
        fps_send_word(cmd->check_sum) ;
        result = 0 ;
    }
    return( result ) ;
}

int fps_recv_response_packet(Response_Packet_Type *res) 
{
    int result = 1 ;
    uint16_t sum = 0 ;
    if (res) {
        sum += fps_recv_word(&(res->head)) ;
        sum += fps_recv_word(&(res->device_id)) ;
        sum += fps_recv_dword(&(res->parameter)) ;
        sum += fps_recv_word(&(res->response)) ;
        fps_recv_word(&(res->check_sum)) ;
        if (res->check_sum == sum) {
            result = 0 ;
        }
    }
    return( result ) ;
}


int  fps_send_data_packet(Data_Packet_Type *_data_packet, int num_data) 
{
    (void) _data_packet ;
    (void) num_data ;
    uint16_t sum = 0 ;    
    int result = 0 ;
    
    sum += fps_send_word(_data_packet->head) ;
    sum += fps_send_word(_data_packet->device_id) ;
    if (_data_packet->data) {
        int i ;
        for (i = 0 ; i < num_data ; i++ ) {
            sum += fps_send_byte(_data_packet->data[i]) ;
        }
    }
    fps_send_word(sum) ; /* send checksum */
    
    fps_recv_response_packet(&_res_packet) ;
    if (_res_packet.response != CMD_ACK) { /* error */
        switch(_res_packet.parameter) {
        case NACK_COMM_ERR:
            result = _res_packet.parameter ;
            UART_PutString("Error: NACK_COMM_ERR\n") ;
            break ;
        case NACK_DEV_ERR:
            result = _res_packet.parameter ;
            UART_PutString("Error: NACK_DEV_ERR\n") ;
            break ;
        default:
            result = _res_packet.parameter ;
            sprintf(str, "Error: Duplicated ID[%d]\n", result) ;
            UART_PutString(str) ;
            break ;
        }
    }

    return( result ) ;
}

int fps_recv_data_packet(Data_Packet_Type *_data_packet, int num_data) 
{
    int result = 1 ;
    uint16_t sum = 0 ;
    if (_data_packet) {
        sum += fps_recv_word(&(_data_packet->head)) ;
        sum += fps_recv_word(&(_data_packet->device_id)) ;
        sum += fps_recv_data(_data_packet->data, num_data) ;
        fps_recv_word(&(_data_packet->check_sum)) ;
        if (_data_packet->check_sum == sum) {
            result = 0 ;
        }
    }
    return( result ) ;
}

int fps_recv_and_dump_packet(Data_Packet_Type *_data_packet, int num_data) 
{
    int result = 1 ;
    uint16_t sum = 0 ;
    if (_data_packet) {
        sum += fps_recv_word(&(_data_packet->head)) ;
        sum += fps_recv_word(&(_data_packet->device_id)) ;
        sum += fps_recv_data(_data_packet->data, num_data) ;
        sum += fps_recv_and_dump_data(num_data) ;
        fps_recv_word(&(_data_packet->check_sum)) ;
        if (_data_packet->check_sum == sum) {
            result = 0 ;
        }
    }
    return(result) ;
}

void fps_print_devinfo(uint8_t buf[])
{
    int i ;
    int index = 0 ;
    uint32_t fwv = 0 ;
    uint32_t isosize = 0 ;
    UART_PutString("Firmware version: ") ;
    fwv = buf[index++] ;
    fwv |= (buf[index++] & 0xFF) <<  8 ;
    fwv |= (buf[index++] & 0xFF) << 16 ;
    fwv |= (buf[index++] & 0xFF) << 24 ;
    sprintf(str, "%08X\n", fwv) ;
    UART_PutString(str) ;
    CyDelay(100) ;
    UART_PutString("IsoAreaMaxSize: ") ;
    isosize = buf[index++] ;
    isosize |= (buf[index++] & 0xFF) <<  8 ;
    isosize |= (buf[index++] & 0xFF) << 16 ;
    isosize |= (buf[index++] & 0xFF) << 24 ;
    sprintf(str, "%08X\n", isosize) ;
    UART_PutString(str) ;  
    CyDelay(100) ;
    UART_PutString("Device Serial Number: ") ;
    for (i = 0 ; i < 16 ; i++ ) {
        sprintf(str, "%02X", buf[index++]) ;
        UART_PutString(str) ;
        CyDelay(10) ;
    }
    UART_PutString("\n") ;
    CyDelay(100) ;
}

void fps_dump_data(uint8_t data[], int num_data)
{
    int i ;
    for (i = 0 ; i < num_data ; i++ ) {
        sprintf(str, "%02X ", data[i]) ;
        UART_PutString(str) ;
        if (((i + 1)%0x10) == 0) {
            UART_PutString("\n") ;
        }
    }
    if (num_data % 0x10) {
        UART_PutString("\n") ;
    }
}

int  fps_open(int extra_info) 
{
    _cmd_packet.command = CMD_OPEN ;
    _cmd_packet.parameter = extra_info ;

    fps_send_command_packet(&_cmd_packet) ;
    CyDelay(100) ;
    fps_recv_response_packet(&_res_packet) ;

    CyDelay(100) ;
    if (_cmd_packet.parameter && (_res_packet.response == CMD_ACK)) {
        _data_packet.data = data_buf ;
        fps_recv_data_packet(&_data_packet, 24) ;
        fps_print_devinfo(data_buf) ;
    }
    return( _res_packet.response ) ;
}

int  fps_close(void) 
{
    _cmd_packet.command = CMD_CLOSE ;
    _cmd_packet.parameter = 0 ;
    fps_send_command_packet(&_cmd_packet) ;
    CyDelay(100) ;
    fps_recv_response_packet(&_res_packet) ;
    return(_res_packet.response) ;
}

int  fps_first_search(void) 
{
    _cmd_packet.command = CMD_USB_INTERNAL_CHECK ;
    fps_send_command_packet(&_cmd_packet) ;
    CyDelay(100) ;
    fps_recv_response_packet(&_res_packet) ;
    if (_res_packet.parameter != 0x55) {
        sprintf(str, "Invalid response paramter 0x%02X (0x55 expected)\n",
            _res_packet.parameter) ;
        UART_PutString(str) ;
    }
    return(_res_packet.response) ;
}

int  fps_led(int led_on) 
{
    _cmd_packet.command = CMD_CMOS_LED ;
    _cmd_packet.parameter = led_on ; /* LED ON */
    fps_send_command_packet(&_cmd_packet) ;
    CyDelay(100) ;
    fps_recv_response_packet(&_res_packet) ;    
    return(_res_packet.response) ;
}

/**
 * fps_change_baudrate
 * @params baudrate 9600, 19200, 38400, 57600, 115200
 * @returns ACK or NACK
 * Note: restore 9600 (default) before leaving the program 
 */
int  fps_change_baudrate(uint32_t baudrate) 
{
    int div = 417 ; /* 9600 baud for 48MHz */
    _cmd_packet.command = CMD_CHANGE_BAURDRATE ;
    switch(baudrate) {
    case   9600: div =     BAUD_COUNT_9600 ; break ;
    case  19200: div =    BAUD_COUNT_19200 ; break ;
    case  38400: div =    BAUD_COUNT_38400 ; break ;
    case  57600: div =    BAUD_COUNT_57600 ; break ;
    case 115200: div =   BAUD_COUNT_115200 ; break ;
    default:
        sprintf(str, "Unsupported baudrate %d, using 9600\n", baudrate) ;
        UART_PutString(str) ;
        baudrate = 9600 ;
        div = 417 ;
        break ;
    }
    _cmd_packet.parameter = baudrate ;
    fps_send_command_packet(&_cmd_packet) ;
    CyDelay(100) ;
    /* we need to change baudrate of FPS to baudrate here */
    FPS_Stop() ;
    FPS_Clock_Stop() ;
    FPS_Clock_SetDivider(div) ;
    FPS_Clock_Start() ;
    FPS_Start() ;
    CyDelay(100) ;
    fps_recv_response_packet(&_res_packet) ;
    return(_res_packet.response) ;
}

int  fps_get_enrolled_count(void) 
{
    _cmd_packet.command = CMD_GET_ENROLL_COUNT ;
    _cmd_packet.parameter = 0 ;
    fps_send_command_packet(&_cmd_packet) ;
    CyDelay(100) ;
    fps_recv_response_packet(&_res_packet) ;
    if (_res_packet.response != CMD_ACK) {
        _res_packet.parameter = -1 ;
    }
    return(_res_packet.parameter) ;
}

/**
 * fps_check_enrolled
 * @param id : id to check if it is enrolled
 * @returns id (if enrolled)
 *   NACK_INVALID_POS (0x1003) if id is not between 0 ~ 999
 *   NACK_IS_NOT_USED (0x1004) if id is not used
 */
int  fps_check_enrolled(int id) 
{
    int result ;
    
    _cmd_packet.command = CMD_CHECK_ENROLLED ;
    _cmd_packet.parameter = id ;    
    fps_send_command_packet(&_cmd_packet) ;
    CyDelay(100) ;
    fps_recv_response_packet(&_res_packet) ;
    if (_res_packet.response != CMD_ACK) {
        result = _res_packet.parameter ;
    } else {
        result = id ; /* return the id */
    }
    return(result) ;
}

/**
 * fps_start_enrollment
 * @param id 0~2999
 * If Parameter's HIWORD is non-zero
 * fingerprint duplication check will not be done
 * if id == -1 then "Enrollment without saving will be performed
 * @returns 0: success non-zero: error
 */
int  fps_start_enrollment(int id, int check_duplication) 
{
    int result = 0 ;
    _cmd_packet.command = CMD_ENROLL_START ;
    if (check_duplication) {
        _cmd_packet.parameter = id & 0x0000FFFF ; /* Hiword = 0 */
    } else {
        _cmd_packet.parameter = id | 0xFFFF0000 ; /* Hiword = 0xFFFF */
    }
    if (id == -1) {
        enroll_without_save = 1 ;
    } else {
        enroll_without_save = 0 ;
    }
    fps_send_command_packet(&_cmd_packet) ;
    CyDelay(1000) ;
    
    fps_recv_response_packet(&_res_packet) ;
    if (_res_packet.response == CMD_ACK) {
        result = 0 ;
    } else if (_res_packet.response != CMD_ACK) {
        result = _res_packet.parameter ;
        switch(result) {
        case NACK_DB_IS_FULL:
            sprintf(str, "Error: Database is full!\n") ;
            break ;
        case NACK_INVALID_POS:
            sprintf(str, "Error: Invalide poisition [%d]\n", id) ;
            break ;
        case NACK_IS_ALREADY_USED:
            sprintf(str, "Error: ID[%d] is already enrolled\n", id) ;
            break ;
        default:
            sprintf(str, "Unknown error 0x%04X\n", result) ;
            break ;
        }
        UART_PutString(str) ;
    }
    return(result) ;
}

/**
 * fps_enroll1
 * @returns 0: success non-zero: error
 */
int fps_enroll1(void)
{
    int result = 0 ;
    _cmd_packet.command = CMD_ENROLL1 ;
    fps_send_command_packet(&_cmd_packet) ;
    CyDelay(2000) ;
    
    fps_recv_response_packet(&_res_packet) ;
    if (_res_packet.response == CMD_ACK) {
        result = 0 ;
    } else {
        result = _res_packet.parameter ;
        switch(result) {
        case NACK_ENROLL_FAILED:
            sprintf(str, "Error: Enroll Failed\n") ;
            break ;
        case NACK_BAD_FINGER:
            sprintf(str, "Error: Bad Finger\n") ;
            break ;
        default:
            sprintf(str, "Unknown error 0x%04X\n", result) ;
            break ;
        }
        UART_PutString(str) ;
    }
    return(result) ;
}

/**
 * fps_enroll2
 * @returns 0: success non-zero: error
 */
int  fps_enroll2(void) 
{
    int result = 0 ;
    _cmd_packet.command = CMD_ENROLL2 ;
    fps_send_command_packet(&_cmd_packet) ;
    CyDelay(2000) ;
    
    fps_recv_response_packet(&_res_packet) ;
    if (_res_packet.response == CMD_ACK) {
        result = 0 ;
    } else {
        result = _res_packet.parameter ;
        switch(result) {
        case NACK_ENROLL_FAILED:
            sprintf(str, "Error: Enroll Failed\n") ;
            break ;
        case NACK_BAD_FINGER:
            sprintf(str, "Error: Bad Finger\n") ;
            break ;
        default:
            sprintf(str, "Unknown error 0x%04X\n", result) ;
            break ;
        }
        UART_PutString(str) ;
    }
    return(result) ;   
}

/**
 * fps_enroll3
 * @returns 0: success non-zero: error
 */
int  fps_enroll3(void) 
{
    int result = 0 ;
    _cmd_packet.command = CMD_ENROLL3 ;
    fps_send_command_packet(&_cmd_packet) ;
    CyDelay(4000) ; // was 4000
    
    fps_recv_response_packet(&_res_packet) ;
    if (_res_packet.response == CMD_ACK) {
        result = 0 ;
    } else {
        result = _res_packet.parameter ;
        switch(result) {
        case NACK_ENROLL_FAILED:
            sprintf(str, "Error: Enroll Failed\n") ;
            break ;
        case NACK_BAD_FINGER:
            sprintf(str, "Error: Bad Finger\n") ;
            break ;
        default:
            if ((0 <= result)&&(result < 3000)) {
                sprintf(str, "Error: Duplicated ID[%d]\n", result) ;
                result |= 0x8000 ; /* flag MSB as an error */
            } else {
                sprintf(str, "Unknown error 0x%04X\n", result) ;
            }
            break ;
        }
        UART_PutString(str) ;
    }
    CyDelay(100) ;

    if (enroll_without_save && (_res_packet.response == CMD_ACK)) {
        _data_packet.data = data_buf ;
        fps_recv_data_packet(&_data_packet, TEMPLATE_SIZE) ;
        fps_dump_data(data_buf, TEMPLATE_SIZE) ;
    }
    return(result) ;  
}

/** 
 * fps_is_finger_pressed
 * This command is used while enrollment,
 * the host waits to take off the finger
 * per enrollment statge 
 * @returns 0: finger is not pressed 1: finger is pressed
 */
int  fps_is_finger_pressed(void) 
{
    int result = 0 ;
    _cmd_packet.command = CMD_IS_PRESS_FINGER ;
    fps_send_command_packet(&_cmd_packet) ;
    CyDelay(100) ;
    
    fps_recv_response_packet(&_res_packet) ;
    if (_res_packet.response == CMD_ACK) {
        if (_res_packet.parameter == 0) {
            result = 1 ; /* Finger is pressed */
        } else {
            result = 0 ; /* Finger is not pressed */
        }
    }
    return(result) ;
}

int  fps_delete_id(int id) 
{
    int result = 0 ;
    _cmd_packet.command = CMD_DELETE_ID ;
    _cmd_packet.parameter = id ;
    fps_send_command_packet(&_cmd_packet) ;
    CyDelay(3000) ;
    
    fps_recv_response_packet(&_res_packet) ;
    if (_res_packet.response != CMD_ACK) {
        result = _res_packet.parameter ;
        if (_res_packet.parameter == NACK_INVALID_POS) {
            sprintf(str, "Error: Invalid id[%d]\n", id) ;
        } else {
            sprintf(str, "Unknown Error [%d]\n", _res_packet.parameter) ;
        }
        UART_PutString(str) ;
    }
    return(result) ;
}

int  fps_delete_all(void) 
{
    int result = 0 ;
    _cmd_packet.command = CMD_DELETE_ALL ;
    fps_send_command_packet(&_cmd_packet) ;
    CyDelay(3000) ;
    
    fps_recv_response_packet(&_res_packet) ;
    if (_res_packet.response != CMD_ACK) {
        result = _res_packet.parameter ;
        if (_res_packet.parameter == NACK_DB_IS_EMPTY) {
            sprintf(str, "Error: Database is empty\n") ;
        } else {
            sprintf(str, "Unknown Error [%d]\n", _res_packet.parameter) ;
        }
        UART_PutString(str) ;
    }
    return(result) ;    
}

int  fps_verify(int id) 
{
    int result = 0 ;
    _cmd_packet.command = CMD_VERIFY ;
    _cmd_packet.parameter = id ;
    fps_send_command_packet(&_cmd_packet) ;
    CyDelay(3000) ;
    
    fps_recv_response_packet(&_res_packet) ;
    if (_res_packet.response != CMD_ACK) {
        result = _res_packet.parameter ;
        switch(result) {
        case NACK_INVALID_POS:
            sprintf(str, "Error: Invalid id[%d]\n", id) ;
            break ;
        case NACK_IS_NOT_USED:
            sprintf(str, "Error: id[%d] is not enrolled\n", id) ;
            break ;
        case NACK_VERIFY_FAILED:
            sprintf(str, "Error: Verify Falied\n") ;
            break ;
        default:
            break ;
            sprintf(str, "Error: Unknown Error [%d]\n", result) ;
            break ;
        }
        UART_PutString(str) ;
    }
    return(result) ;
}

int  fps_identify(void) 
{
    int result = 0 ;
    _cmd_packet.command = CMD_IDENTIFY ;
    fps_send_command_packet(&_cmd_packet) ;
    CyDelay(3000) ;
    
    fps_recv_response_packet(&_res_packet) ;
    if (_res_packet.response == CMD_ACK) {
        result = _res_packet.parameter ; /* identified id */
    } else if (_res_packet.response != CMD_ACK) {
        result = _res_packet.parameter ;
        switch(result) {
        case NACK_DB_IS_EMPTY:
            sprintf(str, "Error: Database is empty\n") ;
            break ;
        case NACK_IDENTIFY_FAILED:
            sprintf(str, "Error: Failed to identify\n") ;
            break ;
        default:
            sprintf(str, "Unknown Error [%d]\n", result) ;
            break ;
        }
        UART_PutString(str) ;
    }
    return(result) ;
}

int  fps_verify_template(int id, uint8_t template[]) 
{
    int result = 0 ;
    int i ;
    _cmd_packet.command = CMD_VERIFY_TEMPLATE ;
    _cmd_packet.parameter = id ;
    fps_send_command_packet(&_cmd_packet) ;
    CyDelay(100) ;
    
    fps_recv_response_packet(&_res_packet) ;   
    if (_res_packet.response == CMD_ACK) {
        for (i = 0 ; i < TEMPLATE_SIZE ; i++ ) {
            FPS_PutChar(template[i]) ;
        }
        CyDelay(100) ;
        fps_recv_response_packet(&_res_packet) ; 
        
        if (_res_packet.response != CMD_ACK) {
            result = _res_packet.parameter ;
            switch(result) {
            case NACK_COMM_ERR:
                sprintf(str, "Error: Communication Error\n") ;
                break ;
            case NACK_VERIFY_FAILED:
                sprintf(str, "Error: Verify failed\n") ;
                break ;
            default:
                sprintf(str, "Unknown Error [%d]\n", result) ;
                break ;
            }
            UART_PutString(str) ;
        }
    } else { 
        result = _res_packet.parameter ;
        switch(result) {
        case NACK_INVALID_POS:
            sprintf(str, "Error: Invalid id[%d]\n", id) ;
            break ;
        case NACK_IS_NOT_USED:
            sprintf(str, "Error: id[%d] is not enrolled\n", id) ;
            break ;
        default:
            sprintf(str, "Unknown Error [%d]\n", result) ;
            break ;
        }
        UART_PutString(str) ;
    }
    return(result) ;
}

int  fps_identify_template(uint8_t template[]) 
{
    int result = 0 ;
    int i ;
    _cmd_packet.command = CMD_IDENTIFY_TEMPLATE ;
    fps_send_command_packet(&_cmd_packet) ;
    CyDelay(100) ;
    
    fps_recv_response_packet(&_res_packet) ;   
    if (_res_packet.response == CMD_ACK) {
        for (i = 0 ; i < TEMPLATE_SIZE ; i++ ) {
            FPS_PutChar(template[i]) ;
        }
        CyDelay(100) ;
        fps_recv_response_packet(&_res_packet) ; 
        
        if (_res_packet.response == CMD_ACK) { /* identify success! */
            result = _res_packet.parameter ; /* identified id */
        } else if (_res_packet.response != CMD_ACK) {
            result = _res_packet.parameter ;
            switch(result) {
            case NACK_COMM_ERR:
                sprintf(str, "Error: Communication Error\n") ;
                break ;
            case NACK_IDENTIFY_FAILED:
                sprintf(str, "Error: Identify failed\n") ;
                break ;
            default:
                sprintf(str, "Unknown Error [%d]\n", result) ;
                break ;
            }
            UART_PutString(str) ;
        }
    } else { 
        result = _res_packet.parameter ;
        switch(result) {
        case NACK_DB_IS_EMPTY:
            sprintf(str, "Error: Database is empty\n") ;
            break ;
        default:
            sprintf(str, "Unknown Error [%d]\n", result) ;
            break ;
        }
        UART_PutString(str) ;
    }
    return(result) ;    
}

/**
 * fps_capture_finger
 * The fingerprint algorithm uses 450dpi 256x256 image for its input.
 * This command captures raw image from the sensor and converts it to 256x256
 * image for the fingerprint algorithm.
 * If the finger is not pressed, this command returns with non-acknowledge.
 * Use best image for enrollment to get the best enrollment data.
 * Use non best image for identification (verification) to get faster
 * user sensibility.
 */
int  fps_capture_finger(int best_image) 
{
    int result = 0 ;
    _cmd_packet.command = CMD_CAPTURE_FINGER ;
    _cmd_packet.parameter = best_image ; /* non-zero for best image */
    fps_send_command_packet(&_cmd_packet) ;
    CyDelay(3000) ;
    
    fps_recv_response_packet(&_res_packet) ; 
    if (_res_packet.response != CMD_ACK) {
        result = _res_packet.parameter ;
        switch(result) {
        case NACK_FINGER_IS_NOT_PRESSED:
            sprintf(str, "Error: Finger is not pressed\n") ;
            break ;
        default:
            sprintf(str, "Error: Unknown Error [%d]\n", result) ;
            break ;
        }
        UART_PutString(str) ;
    }
    return(result) ;
}

int  fps_make_template(void) 
{
    int result = 0 ;
    _cmd_packet.command = CMD_MAKE_TEMPLATE ;
    fps_send_command_packet(&_cmd_packet) ;
    CyDelay(100) ;
    
    fps_recv_response_packet(&_res_packet) ;     
    if (_res_packet.response == CMD_ACK) {
        fps_recv_data(data_buf, TEMPLATE_SIZE) ;
        fps_dump_data(data_buf, TEMPLATE_SIZE) ;
    } else if (_res_packet.response != CMD_ACK) {
        result = _res_packet.parameter ;
        switch(result) {
        case NACK_BAD_FINGER:
            sprintf(str, "Error: Bad Finger\n") ;
            break ;
        default:
            sprintf(str, "Unknown Error [%d]\n", result) ;
            break ;
        }
        UART_PutString(str) ;
    }
    return(result) ;
}

int  fps_get_image(void) 
{
    int result = 0 ;
    _cmd_packet.command = CMD_GET_IMAGE ;
    fps_send_command_packet(&_cmd_packet) ;
    CyDelay(100) ;
    
    fps_recv_response_packet(&_res_packet) ; 
    if (_res_packet.response == CMD_ACK) {
        result = fps_recv_and_dump_data(52116) ; /* 258 x 202 = 52116 bytes */
    }
    return(result) ;
}

int  fps_get_raw_image(void) 
{
    int result = 0 ;
    _cmd_packet.command = CMD_GET_RAW_IMAGE ;
    fps_send_command_packet(&_cmd_packet) ;
    CyDelay(100) ;
    
    fps_recv_response_packet(&_res_packet) ; 
    if (_res_packet.response == CMD_ACK) {
        result = fps_recv_and_dump_data(19200) ; /* 160x120 = 19200 bytes */
    }
    return(result) ;
}

int  fps_get_template(int id, uint8_t template[]) 
{
    int result = 0 ;
    _cmd_packet.command = CMD_GET_TEMPLATE ;
    _cmd_packet.parameter = id ;
    fps_send_command_packet(&_cmd_packet) ;
    CyDelay(100) ;
    
    fps_recv_response_packet(&_res_packet) ;  
    if (_res_packet.response == CMD_ACK) {
        result = fps_recv_data(template, TEMPLATE_SIZE) ;
    }
    return(result) ;
}

int  fps_set_template(int id, uint8_t template[], int check_duplication) 
{
    int result = 0 ;
    _cmd_packet.command = CMD_SET_TEMPLATE ;

    if (check_duplication) {
        _cmd_packet.parameter = 0x0000FFFF & id ;
    } else {
        _cmd_packet.parameter = 0xFFFF0000 | id ;
    }
    fps_send_command_packet(&_cmd_packet) ;
    CyDelay(100) ;
    
    fps_recv_response_packet(&_res_packet) ;  
    if (_res_packet.response == CMD_ACK) {
        _data_packet.data = template ;
        result = fps_send_data_packet(&_data_packet, TEMPLATE_SIZE) ;
    }
    return(result) ;    
}

// int  fps_download_database(void) {}   /* obsolete, do nothing */
// int  fps_end_download(void) {}  /* obsolete, do nothing */

int  fps_set_security_level(int level) 
{
    int result = 0 ;
    
    _cmd_packet.command = CMD_SET_SECURITY_LEVEL ;
    _cmd_packet.parameter = level ;
    fps_send_command_packet(&_cmd_packet) ;
    CyDelay(100) ;
    
    fps_recv_response_packet(&_res_packet) ;
    if (_res_packet.response != CMD_ACK) {
        UART_PutString("Error: set security level failed\n") ;
        result = CMD_NACK ;
    }
    return( result ) ;
}

int  fps_get_security_level(void) 
{
    int level = 0 ;
    
    _cmd_packet.command = CMD_GET_SECURITY_LEVEL ;
    fps_send_command_packet(&_cmd_packet) ;
    CyDelay(100) ;
    
    fps_recv_response_packet(&_res_packet) ;
    if (_res_packet.response == CMD_ACK) {
        level = _res_packet.parameter ;
    } else {
        UART_PutString("Error: get security level failed\n") ;
    }
    return( level ) ;
}

int  fps_enter_stadby(void) 
{
    int result = 0 ;
    
    _cmd_packet.command = CMD_ENTER_STANDBY_MODE ;
    fps_send_command_packet(&_cmd_packet) ;
    CyDelay(100) ;
    
    fps_recv_response_packet(&_res_packet) ;
    if (_res_packet.response != CMD_ACK) {
        UART_PutString("Error: enter stadnby mode failed\n") ;
        result = _res_packet.response ;
    }
    return( result ) ;
}

int  fps_wake_up(void)  /* send 0x00 and wait 20ms */
{
    int result = 0 ;
    FPS_PutChar(0x00) ;
    CyDelay(20) ;
    return( result ) ;
}

/**
 * fps_identify_template2
 * Identify of the capture fingerprint image
 * with the specified template
 */
int  fps_identify_template2(uint8_t template[]) 
{
    int result = 0 ;
    _cmd_packet.command = CMD_IDENTIFY_TEMPLATE2 ;
    _cmd_packet.parameter = 500 ;
    fps_send_command_packet(&_cmd_packet) ;
    CyDelay(100) ;
    
    fps_recv_response_packet(&_res_packet) ;
    if (_res_packet.response == CMD_ACK) {
        int i ;
        data_buf[0] = 1 ;
        data_buf[1] = 1 ;
        for (i = 0 ; i < TEMPLATE_SIZE ; i++ ) {
            data_buf[i+2] = template[i] ;
        }
        fps_send_command_packet(&_cmd_packet) ;
        
        CyDelay(100) ;
        fps_recv_response_packet(&_res_packet) ;
        if (_res_packet.response != CMD_ACK) {
            result = _res_packet.response ;
            if (_res_packet.parameter == NACK_IDENTIFY_FAILED) {
                UART_PutString("Error: Identify failed\n") ;
            } else {
                UART_PutString("Error: identify template2 failed\n") ;
            }
        }   
    } else {
        result = _res_packet.response ;
        UART_PutString("Sending identify template2 command failed\n") ;
    }
    return(result) ;
}

/* [] END OF FILE */
